/*
Copyright (C) 2001 Chr. Clemens Lee <clemens@kclee.com>.

This file is part of SimplEdit
(http://www.kclee.org/clemens/java/simpledit/).

SimplEdit is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

SimplEdit is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with SimplEdit; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

package com.kclee.simpledit;

import ccl.swing.MainJFrame;
import ccl.swing.MainJMenuBar;
import ccl.swing.RunnableAction;
import ccl.swing.SwingUtil;
import ccl.util.FileUtil;
import ccl.util.Util;
import java.awt.BorderLayout;
import java.awt.Event;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.IOException;
import java.util.Vector;
import javax.swing.JFileChooser;
import javax.swing.JMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;
import javax.swing.KeyStroke;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

/**
 *
 *
 * @author Chr. Clemens Lee
 * @version $Id: ViewController.java,v 1.2 2001/12/30 18:45:43 clemens Exp $
 */
public class ViewController extends MainJFrame
                            implements ActionListener
{
    private static final String S_INI_CONTENT = "[Init]\n"
                                                + "Author=Chr. Clemens Lee\n"
                                                + "JavadocAuthor=Chr. Clemens Lee\n"
                                                + "ProjectSuffix=\n"
                                                + "\n"
                                                + "[Colors]\n"
                                                + "UseSystemColors=true\n";
    private static final String S_MF_NEW     = "&New";
    private static final String S_MF_OPEN    = "&Open...";
    private static final String S_MF_SAVE    = "&Save";
    private static final String S_MF_SAVE_AS = "Save &As...";

    private static final String S_TITLE = "SimplEdit";
    private static final String S_MODIFIED_TITLE = " *";

    private JTextArea _pTextArea = null;

    private void _setTitle() {
	setTitle( "SimplEdit" );
    }

    protected String _getProjectSuffix()
    {
        // allow all kind of suffixes for an editor
        return "";
    }

    public void newFile() {
        if ( !_confirmClose() ) {
            _pTextArea.requestFocus();

            return;
        }

        _pTextArea.setText( "" );

        // do what super would normally do
        getInit().setFileFullName( "" );
        setModified( false );
        setTitle();

        _pTextArea.requestFocus();
    }

    public void open() {
        if ( !_confirmClose() ) {
            _pTextArea.requestFocus();

            return;
        }

        String sProjectFullName = null;
        String sOldProjectPath = getInit().getFilePath().replace('/', File.separatorChar);

        /*
        sProjectFullName = SwingUtil.getFileName
               ( this, 
                 "Choose a Project", 
                 asProjectSuffixes,
                 sOldProjectPath );
                 // */

        // --
        JFileChooser pFileChooser = new JFileChooser
               ( sOldProjectPath );
        pFileChooser.setDialogTitle( "Open File" );
        pFileChooser.setApproveButtonText( "Open" );
        pFileChooser.setFileSelectionMode( JFileChooser.FILES_ONLY );
        int retVal = pFileChooser.showDialog( this, null );
        if ( retVal != JFileChooser.APPROVE_OPTION ) {
            return;
        }
        sProjectFullName = pFileChooser.getSelectedFile().getAbsolutePath();
        // --

        if (sProjectFullName == null) {
            return;
        }
        
        open( sProjectFullName );
    }

    public void open( String sFileName_ ) {
        if ( !FileUtil.exists( sFileName_ ) )
        {
            _pTextArea.setText( "" );
        }
        else
        {
            try
            {
                if ( !FileUtil.isAbsolute( sFileName_ ) )
                {
                    sFileName_ = FileUtil.getAbsoluteFileName( FileUtil.concatPath( (String)System.getProperties().get( "user.dir" ), sFileName_ ) );
                }
                _pTextArea.setText( FileUtil.readFile( sFileName_ ) );
            }
            catch( IOException ioException )
            {
                SwingUtil.showMessage( this
                                       , "Error reading file:\n'"
                                       + sFileName_
                                       + "'" );
                Util.printlnErr( Util.getStackTrace( ioException ) );
            }
        }

        // do what super would normally do
        getInit().setFileFullName( sFileName_ );
        setModified( false );
        setTitle();


        _pTextArea.requestFocus();
    }

    public void save()
    {
        String sTempFileName = getInit().getFileFullName();

        if ( Util.isEmpty( sTempFileName ) ) {
            saveAs();

            return;
        }

        _backup    ( sTempFileName );

        save       ( sTempFileName );

        setModified( false         );
        setTitle   ();

        _pTextArea.requestFocus();
    }

    /**
     * Implement the logic to save your application project
     * data in this method.
     */
    public void save( String sFileName_ ) {
        try
        {
            if ( !FileUtil.isAbsolute( sFileName_ ) )
            {
                sFileName_ = FileUtil.getAbsoluteFileName( FileUtil.concatPath( (String)System.getProperties().get( "user.dir" ), sFileName_ ) );
            }
            FileUtil.writeFile( sFileName_, _pTextArea.getText() );
        }
        catch( IOException ioException )
        {
            SwingUtil.showMessage( this
                                   , "Error writing file:\n'"
                                     + sFileName_
                                     + "'" );
            Util.printlnErr( Util.getStackTrace( ioException ) );
        }

        _pTextArea.requestFocus();
    }

    public void saveAs() {
        // find file name
        String sProjectSuffix = _getProjectSuffix();
        String sOldProjectPath = getInit().getFilePath().replace('/', File.separatorChar);
        /*String sFileName = SwingUtil.getFileName
               ( this, 
                 "Save As",
                 ".txt",
                 sOldProjectPath,
                 "Save As" );
        // */
        // --
        JFileChooser pFileChooser = new JFileChooser
               ( sOldProjectPath );
        pFileChooser.setDialogTitle( "Save As" );
        pFileChooser.setApproveButtonText( "Save As" );
        pFileChooser.setFileSelectionMode( JFileChooser.FILES_ONLY );
        int retVal = pFileChooser.showDialog( this, null );
        if ( retVal != JFileChooser.APPROVE_OPTION ) {
            return;
        }
        String sFileName = pFileChooser.getSelectedFile().getAbsolutePath();
        // --

        if ( sFileName == null) {
            return;
        }

        if ( FileUtil.existsDir( sFileName ) ) {
            SwingUtil.showMessage( this, 
                                   "Sorry, file\n\n'" +
                                   sFileName +
                                   "'\n\nis a directory." +
                                   "Please choose another file name to save your data." );

            // try it again
            saveAs();
                
            return;
        }
        if ( FileUtil.existsFile( sFileName ) ) {
            boolean bOK = SwingUtil.isOKOrCancel( this, 
                                                  "File\n\n'" +
                                                  sFileName +
                                                  "'\n\nexists already. Is it OK to overwrite it?." );
            if ( !bOK ) {
                return;
            }
        }   
        getInit().setFileFullName( sFileName );

        save();
    }

    public class NewAction extends RunnableAction {
        public NewAction() {
            super( "&New" );
        }

        public void run() {
            newFile();
        }

        public KeyStroke getAccelerator() {
            return KeyStroke.getKeyStroke( KeyEvent.VK_N,
                                           Event.CTRL_MASK );
        }
    }

    public class OpenAction extends RunnableAction {
        public OpenAction() {
            super( "&Open..." );
        }

        public void run() {
            open();
        }

        public KeyStroke getAccelerator() {
            return KeyStroke.getKeyStroke( KeyEvent.VK_O,
                                           Event.CTRL_MASK );
        }
    }

    private class SaveAction extends RunnableAction {
        public SaveAction() {
            super( "&Save" );
        }

        public void run() {
            save();
        }

        public KeyStroke getAccelerator() {
            return KeyStroke.getKeyStroke( KeyEvent.VK_S,
                                           Event.CTRL_MASK );
        }
    }

    private class SaveAsAction extends RunnableAction {
        public SaveAsAction() {
            super( "Save &As..." );
        }

        public void run() {
            saveAs();
        }
    }

    private void _setMenuBar() {
	Vector vMenus = new Vector();
	Vector vFileMenu = new Vector();
	Vector vHelpMenu = new Vector();

	vFileMenu.addElement( "File" );
	vFileMenu.addElement( new NewAction() );
	vFileMenu.addElement( new OpenAction() );
	vFileMenu.addElement( new SaveAction() );
	vFileMenu.addElement( new SaveAsAction() );
	vFileMenu.addElement( "E&xit" );

	vHelpMenu.addElement( "Help" );
	vHelpMenu.addElement( "&Contents..." );
	vHelpMenu.addElement( "-----" );
	vHelpMenu.addElement( "&About..." );

	vMenus.addElement( vFileMenu );
	vMenus.addElement( vHelpMenu );

	setMenuBar( vMenus );

        // remove old files menu
        JMenu fileMenu = getJMenuBar().getMenu( 0 );
        int menuCount = fileMenu.getMenuComponentCount();
        int delete = menuCount - 6;
        for( int item = 4; delete > 0; item++, delete-- )
        {
            fileMenu.remove( 4 );
        }

        Util.debug( this, "_setMenuBar().EXIT" );
    }

    public ViewController( String[] asArg_, String sRCSHeader_ ) {
        super( "Notepad"
               , asArg_
               , sRCSHeader_
               , S_INI_CONTENT );
        Util.debug( this, "<init>(..).ENTER" );

        if ( isExitSet() ) {
            return;
        }

        _setTitle();
        _setMenuBar();

        _pTextArea = new JTextArea();
        _pTextArea.setFont( new Font( "Monospaced", Font.PLAIN, 12 ) );

	_pTextArea.getDocument().addDocumentListener( new DocumentListener() 
            {
                public void insertUpdate( DocumentEvent e ) {
                    ViewController.this.setModified( true );
                }
                public void removeUpdate( DocumentEvent e ) {
                    ViewController.this.setModified( true );
                }
                public void changedUpdate( DocumentEvent e ) {
                    ViewController.this.setModified( true );
                }
            } );

        JScrollPane scpTextArea = new JScrollPane( _pTextArea );
        scpTextArea.setBorder( SwingUtil.createCCLBorder() );

        getContentPane().setLayout( new BorderLayout() );
        getContentPane().add( scpTextArea, BorderLayout.CENTER );

        Util.debug( this, "<init>(..).BEFORE PACK" );
        pack();
        SwingUtil.maximizeWindow( this );
        Util.debug( this, "<init>(..).BEFORE SHOW" );

        show();
        SwingUtilities.invokeLater( new Runnable() {
            public void run() {
                SwingUtil.maximizeWindow( ViewController.this );
                Util.sleep( 1 );
                SwingUtil.maximizeWindow( ViewController.this );
            }
        } );

        if ( getInit().getArguments().size() > 0 )
        {
            Vector vArguments = getInit().getArguments();
            String sInitialFile = (String)vArguments.elementAt( vArguments.size() - 1 );
            sInitialFile = FileUtil.normalizeFileName( sInitialFile );
            open( sInitialFile );
        }

        _pTextArea.requestFocus();

        Util.debug( this, "<init>(..).EXIT" );
    }

    public void setTitle() {
        // get application name
        String sTitle = S_TITLE;

        // get project name
        String sProjectName = getInit().getFileFullName();
        if ( !Util.isEmpty( sProjectName ) ) {
            sProjectName = sProjectName.substring( sProjectName.lastIndexOf( File.separatorChar ) + 1 );
            sTitle += ": " + sProjectName;
        }

        // is modified?
        if ( isModified() ) {
            sTitle += S_MODIFIED_TITLE;
        }

        // set it
        setTitle( sTitle );
    }
}
